package ru.dbotthepony.kstarbound.gl

import org.lwjgl.opengl.GL46.*
import java.io.Closeable

class GLVertexArrayObject(val state: GLStateTracker) : Closeable {
	val pointer = glGenVertexArrays()

	fun bind(): GLVertexArrayObject {
		check(isValid) { "Tried to use NULL GLVertexArrayObject" }
		return state.bind(this)
	}

	fun unbind(): GLVertexArrayObject {
		check(isValid) { "Tried to use NULL GLVertexArrayObject" }
		return state.unbind(this)
	}

	fun attribute(position: Int, size: Int, type: Int, normalize: Boolean, stride: Int, offset: Long = 0L): GLVertexArrayObject {
		check(isValid) { "Tried to use NULL GLVertexArrayObject" }
		state.ensureSameThread()
		glVertexAttribPointer(position, size, type, normalize, stride, offset)
		checkForGLError()
		return this
	}

	fun enableAttribute(position: Int): GLVertexArrayObject {
		check(isValid) { "Tried to use NULL GLVertexArrayObject" }
		state.ensureSameThread()
		glEnableVertexArrayAttrib(pointer, position)
		//glEnableVertexAttribArray(position)
		checkForGLError()
		return this
	}

	var isValid = true
		private set

	override fun close() {
		state.ensureSameThread()
		if (isValid) return

		if (state.VAO == this) {
			state.VAO = null
		}

		glDeleteVertexArrays(pointer)
		isValid = false
	}
}
